package com.sun.jmx.mbeanserver;

import javax.management.InstanceAlreadyExistsException;
import java.security.AccessController;
import java.lang.reflect.InvocationHandler;
import javax.management.openmbean.OpenDataException;
import javax.management.MBeanServerInvocationHandler;
import java.lang.reflect.Proxy;
import javax.management.JMX;
import java.lang.ref.WeakReference;
import java.util.Map;
import javax.management.ObjectName;
import javax.management.MBeanServerConnection;

public class MXBeanLookup
{
    private static final ThreadLocal<MXBeanLookup> currentLookup;
    private final MBeanServerConnection mbsc;
    private final WeakIdentityHashMap<Object, ObjectName> mxbeanToObjectName;
    private final Map<ObjectName, WeakReference<Object>> objectNameToProxy;
    private static final WeakIdentityHashMap<MBeanServerConnection, WeakReference<MXBeanLookup>> mbscToLookup;
    
    private MXBeanLookup(MBeanServerConnection mbsc) {
        this.mxbeanToObjectName = WeakIdentityHashMap.make();
        this.objectNameToProxy = Util.newMap();
        this.mbsc = mbsc;
    }
    
    static MXBeanLookup lookupFor(MBeanServerConnection mBeanServerConnection) {
        synchronized (MXBeanLookup.mbscToLookup) {
            WeakReference<Object> weakReference = (WeakReference)MXBeanLookup.mbscToLookup.get(mBeanServerConnection);
            MXBeanLookup referent = (weakReference == null) ? null : ((MXBeanLookup)weakReference.get());
            if (referent == null) {
                referent = new MXBeanLookup(mBeanServerConnection);
                MXBeanLookup.mbscToLookup.put(mBeanServerConnection, new WeakReference(referent));
            }
            return referent;
        }
    }
    
    synchronized <T> T objectNameToMXBean(ObjectName objectName, Class<T> interfaceClass) {
        WeakReference<Object> weakReference = (WeakReference)this.objectNameToProxy.get(objectName);
        if (weakReference != null) {
            Object value = weakReference.get();
            if (interfaceClass.isInstance(value)) {
                return interfaceClass.cast(value);
            }
        }
        Object mxBeanProxy = JMX.newMXBeanProxy(this.mbsc, objectName, interfaceClass);
        this.objectNameToProxy.put(objectName, new WeakReference(mxBeanProxy));
        return mxBeanProxy;
    }
    
    synchronized ObjectName mxbeanToObjectName(Object proxy) throws OpenDataException {
        String str;
        if ((proxy instanceof Proxy)) {
            InvocationHandler invocationHandler = Proxy.getInvocationHandler(proxy);
            if ((invocationHandler instanceof MBeanServerInvocationHandler)) {
                MBeanServerInvocationHandler mBeanServerInvocationHandler = (MBeanServerInvocationHandler)invocationHandler;
                if (mBeanServerInvocationHandler.getMBeanServerConnection().equals(this.mbsc)) {
                    return mBeanServerInvocationHandler.getObjectName();
                }
                str = "proxy for a different MBeanServer";
            }
            else {
                str = "not a JMX proxy";
            }
        }
        else {
            ObjectName objectName = (ObjectName)this.mxbeanToObjectName.get(proxy);
            if (objectName != null) {
                return objectName;
            }
            str = "not an MXBean registered in this MBeanServer";
        }
        throw new OpenDataException("Could not convert " + ((proxy == null) ? "null" : ("object of type " + proxy.getClass().getName())) + " to an ObjectName: " + str);
    }
    
    synchronized void addReference(ObjectName objectName, Object o) throws InstanceAlreadyExistsException {
        ObjectName obj = (ObjectName)this.mxbeanToObjectName.get(o);
        if (obj != null && !"true".equalsIgnoreCase((String)AccessController.doPrivileged(new GetPropertyAction("jmx.mxbean.multiname")))) {
            throw new InstanceAlreadyExistsException("MXBean already registered with name " + obj);
        }
        this.mxbeanToObjectName.put(o, objectName);
        return;
    }
    
    synchronized boolean removeReference(ObjectName objectName, Object o) {
        if (objectName.equals(this.mxbeanToObjectName.get(o))) {
            this.mxbeanToObjectName.remove(o);
            return true;
        }
        return false;
    }
    
    static MXBeanLookup getLookup() {
        return (MXBeanLookup)MXBeanLookup.currentLookup.get();
    }
    
    static void setLookup(MXBeanLookup value) {
        MXBeanLookup.currentLookup.set(value);
    }
    
    static {
        currentLookup = new ThreadLocal();
        mbscToLookup = WeakIdentityHashMap.make();
    }
}
